"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _get = _interopRequireDefault(require("lodash/get"));

var _cloneDeep = _interopRequireDefault(require("lodash/cloneDeep"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

var _react = _interopRequireDefault(require("react"));

var _propTypes = _interopRequireDefault(require("prop-types"));

var _ComponentWrapper = _interopRequireDefault(require("./ComponentWrapper"));

var _NotFoundComponent = _interopRequireDefault(require("../NotFoundComponent"));

var _Placeholder = _interopRequireDefault(require("./Placeholder"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { "@babel/helpers - typeof"; if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") { _typeof = function _typeof(obj) { return typeof obj; }; } else { _typeof = function _typeof(obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; } return _typeof(obj); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } }

function _createClass(Constructor, protoProps, staticProps) { if (protoProps) _defineProperties(Constructor.prototype, protoProps); if (staticProps) _defineProperties(Constructor, staticProps); return Constructor; }

function _possibleConstructorReturn(self, call) { if (call && (_typeof(call) === "object" || typeof call === "function")) { return call; } return _assertThisInitialized(self); }

function _getPrototypeOf(o) { _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) { return o.__proto__ || Object.getPrototypeOf(o); }; return _getPrototypeOf(o); }

function _assertThisInitialized(self) { if (self === void 0) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function"); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, writable: true, configurable: true } }); if (superClass) _setPrototypeOf(subClass, superClass); }

function _setPrototypeOf(o, p) { _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) { o.__proto__ = p; return o; }; return _setPrototypeOf(o, p); }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var constants = require('../../commons/constants'); // ToDo: remove once the WebpackDevServer fixes HMR - it should preserve React components state (now it does not)
// https://github.com/webpack/webpack-dev-server/issues/1377
// https://github.com/gaearon/react-hot-loader/issues/934


var storeComponentsTree;

function sendMessage(message) {
  if (message) {
    window.parent.postMessage(message, '*');
  }
}

var handleComponentEvent = function handleComponentEvent(eventName) {
  return function (args) {
    sendMessage({
      type: constants.FRAMEWORK_MESSAGE_COMPONENT_EVENT,
      payload: {
        eventName: eventName,
        args: args,
        timestamp: Date.now()
      }
    });
  };
};

var renderComponent = function renderComponent(userComponents, description, rootProps) {
  if (description) {
    var type = description.type,
        key = description.key,
        props = description.props,
        children = description.children;

    if (!type || !props) {
      return rootProps;
    }

    var componentName = props.componentName,
        propertyName = props.propertyName,
        propertyValue = props.propertyValue;

    if (type === constants.COMPONENT_PROPERTY_ELEMENT_TYPE) {
      var newElement = _react.default.createElement(_Placeholder.default, {
        key: key
      });

      if (rootProps) {
        if (propertyName) {
          rootProps[propertyName] = newElement;
        } else {
          rootProps.push(newElement);
        }
      } else {
        // only placeholder component can be the root element
        rootProps = newElement;
      }
    } else if (type === constants.PAGE_COMPONENT_TYPE || type === constants.PAGE_NODE_TYPE) {
      var _newElement;

      var component = (0, _get.default)(userComponents, componentName, null);

      if (component) {
        var propsComponent = {};

        if (children && children.length > 0) {
          children.forEach(function (child) {
            propsComponent = renderComponent(userComponents, child, propsComponent);
          });
        }

        var nestedComponents = [];

        if (propsComponent.children && (0, _isArray.default)(propsComponent.children)) {
          nestedComponents = propsComponent.children;
          delete propsComponent.children;
        }

        _newElement = _react.default.createElement(_ComponentWrapper.default, {
          wrappedComponent: component,
          propsComponent: propsComponent,
          key: key
        }, nestedComponents);
      } else {
        _newElement = _react.default.createElement(_NotFoundComponent.default, {
          componentName: componentName
        });
      }

      if (rootProps) {
        if (propertyName) {
          // component assigned to some named property in the
          rootProps[propertyName] = _newElement;
        } else {
          rootProps.push(_newElement);
        }
      } else {
        // only page component can be the root element
        rootProps = _newElement;
      }
    } else if (type === constants.COMPONENT_PROPERTY_ARRAY_TYPE || type === constants.COMPONENT_PROPERTY_OBJECT_TYPE) {
      if (rootProps) {
        if (propertyName) {
          if (propertyValue) {
            rootProps[propertyName] = (0, _cloneDeep.default)(propertyValue);
          } else {
            rootProps[propertyName] = propertyValue;
          }
        } else {
          if (typeof propertyValue !== 'undefined') {
            rootProps.push((0, _cloneDeep.default)(propertyValue));
          }
        }
      }
    } else if (type === constants.COMPONENT_PROPERTY_SHAPE_TYPE) {
      var newObject = {};

      if (children && children.length > 0) {
        children.forEach(function (child) {
          newObject = renderComponent(userComponents, child, newObject);
        });
      }

      if (rootProps) {
        if (propertyName) {
          rootProps[propertyName] = newObject;
        } else {
          rootProps.push(newObject);
        }
      }
    } else if (type === constants.COMPONENT_PROPERTY_FUNCTION_TYPE) {
      if (rootProps && propertyName) {
        rootProps[propertyName] = handleComponentEvent(propertyName);
      }
    } else if (type === constants.COMPONENT_PROPERTY_ARRAY_OF_TYPE) {
      var newArrayModel = [];

      if (children && children.length > 0) {
        children.forEach(function (child) {
          newArrayModel = renderComponent(userComponents, child, newArrayModel);
        });
      }

      if (rootProps) {
        if (propertyName) {
          rootProps[propertyName] = newArrayModel;
        } else {
          rootProps.push(newArrayModel);
        }
      }
    } else if (type === constants.COMPONENT_PROPERTY_STRING_TYPE || type === constants.COMPONENT_PROPERTY_ONE_OF_TYPE || type === constants.COMPONENT_PROPERTY_SYMBOL_TYPE || type === constants.COMPONENT_PROPERTY_BOOL_TYPE || type === constants.COMPONENT_PROPERTY_ANY_TYPE || type === constants.COMPONENT_PROPERTY_NUMBER_TYPE) {
      if (rootProps) {
        if (propertyName) {
          if (propertyName !== constants.COMPONENT_PROPERTY_DO_NOT_USE_IN_FLOWS_NAME) {
            rootProps[propertyName] = propertyValue;
          }
        } else {
          if (typeof propertyValue !== 'undefined') {
            if (rootProps.push) {
              rootProps.push(propertyValue);
            } else {
              console.error('It seems that you missed propertyName in the page component target in the page config.');
            }
          }
        }
      }
    }
  }

  return rootProps;
};

var ComponentComposer =
/*#__PURE__*/
function (_React$Component) {
  _inherits(ComponentComposer, _React$Component);

  function ComponentComposer(props) {
    var _this;

    _classCallCheck(this, ComponentComposer);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(ComponentComposer).call(this, props));
    _this.renderPage = _this.renderPage.bind(_assertThisInitialized(_this));
    _this.handleReceiveMessage = _this.handleReceiveMessage.bind(_assertThisInitialized(_this));
    _this.state = {
      componentsTree: storeComponentsTree || {}
    };
    return _this;
  }

  _createClass(ComponentComposer, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      window.addEventListener("message", this.handleReceiveMessage, false);
    }
  }, {
    key: "componentWillUnmount",
    value: function componentWillUnmount() {
      window.removeEventListener("message", this.handleReceiveMessage);
    }
  }, {
    key: "shouldComponentUpdate",
    value: function shouldComponentUpdate(nextProps, nextState) {
      var componentsTree = this.state.componentsTree;
      return componentsTree !== nextState.componentsTree;
    }
  }, {
    key: "componentDidUpdate",
    value: function componentDidUpdate(prevProps, prevState, snapshot) {
      var componentsTree = this.state.componentsTree;

      if (componentsTree !== prevState.componentsTree) {
        storeComponentsTree = componentsTree;
      }
    }
  }, {
    key: "handleReceiveMessage",
    value: function handleReceiveMessage(event) {
      var message = event.data;

      if (message) {
        var type = message.type,
            payload = message.payload;

        if (type === constants.WEBCODESK_MESSAGE_UPDATE_PAGE_COMPONENTS_TREE) {
          this.setState({
            componentsTree: payload
          });
        }
      }
    }
  }, {
    key: "renderPage",
    value: function renderPage() {
      var userComponents = this.props.userComponents;
      var componentsTree = this.state.componentsTree;
      return renderComponent(userComponents, componentsTree);
    }
  }, {
    key: "render",
    value: function render() {
      var content = this.renderPage();

      if (content) {
        return content;
      }

      return _react.default.createElement("span", null);
    }
  }]);

  return ComponentComposer;
}(_react.default.Component);

_defineProperty(ComponentComposer, "propTypes", {
  userComponents: _propTypes.default.object
});

_defineProperty(ComponentComposer, "defaultProps", {
  userComponents: {}
});

var _default = ComponentComposer;
exports.default = _default;