"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clearActionsCache = clearActionsCache;
exports.default = _default;

var _cloneDeep = _interopRequireDefault(require("lodash/cloneDeep"));

var _forOwn = _interopRequireDefault(require("lodash/forOwn"));

var _constants = require("./constants");

var _sequences = require("./sequences");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _toConsumableArray(arr) { return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread(); }

function _nonIterableSpread() { throw new TypeError("Invalid attempt to spread non-iterable instance"); }

function _iterableToArray(iter) { if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter); }

function _arrayWithoutHoles(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } }

var sendDebugMessage = require('../commons/sendMessage').default;

var constants = require('../commons/constants');

function dispatchToComponent(props, payload, dispatch) {
  if (props) {
    var componentName = props.componentName,
        componentInstance = props.componentInstance,
        componentKey = props.componentKey;
    var targetKey = "".concat(componentName, "_").concat(componentInstance);

    if (window.__webcodeskIsListeningToFramework && window.__sendFrameworkMessage) {
      sendDebugMessage({
        key: componentKey,
        eventType: constants.DEBUG_MSG_REDUCE_DATA_EVENT,
        inputData: (0, _cloneDeep.default)(payload),
        componentName: componentName,
        componentInstance: componentInstance,
        timestamp: Date.now()
      });
    }

    dispatch({
      type: targetKey,
      payload: payload
    });
  }
}

function findEventTargets(events, type) {
  var result;

  if (events && events.length > 0) {
    var event = events.find(function (targetEvent) {
      return targetEvent.name === type;
    });

    if (event && event.targets && event.targets.length > 0) {
      result = event.targets;
    }
  }

  return result;
}

function executeUserFunctionDispatch(events, innerTasks, functionKey, functionName, dispatchPayloads, dispatch, getState, helpers) {
  var dispatchTypeKeys = dispatchPayloads ? Object.keys(dispatchPayloads) : null;
  var dispatchType;
  var payload;

  if (dispatchTypeKeys && dispatchTypeKeys.length > 0) {
    for (var i = 0; i < dispatchTypeKeys.length; i++) {
      dispatchType = dispatchTypeKeys[i]; // check if the user function dispatches any event

      var eventTargets = findEventTargets(events, dispatchType);

      if (eventTargets && eventTargets.length > 0) {
        payload = dispatchPayloads[dispatchType];

        if (window.__webcodeskIsListeningToFramework && window.__sendFrameworkMessage) {
          sendDebugMessage({
            key: functionKey,
            eventType: constants.DEBUG_MSG_FUNCTION_FIRE_EVENT,
            eventName: dispatchType,
            outputData: (0, _cloneDeep.default)(payload),
            functionName: functionName,
            timestamp: Date.now()
          });
        }

        eventTargets.forEach(function (eventTarget) {
          var eventTargetType = eventTarget.type,
              eventTargetProps = eventTarget.props;

          if (eventTargetType === _constants.COMPONENT_TYPE) {
            dispatchToComponent(eventTargetProps, payload, dispatch);
          }
        });

        if (innerTasks[dispatchType] && innerTasks[dispatchType].length > 0) {
          innerTasks[dispatchType].forEach(function (task) {
            var func = task.func;
            func.apply(null, [payload])(dispatch, getState, helpers);
          });
        }
      }
    }
  }
}

function createTasks(targets) {
  var tasks = [];

  if (targets && targets.length > 0) {
    targets.forEach(function (target) {
      var type = target.type,
          props = target.props,
          events = target.events;

      if (type === _constants.USER_FUNCTION_TYPE && props) {
        var _func = (0, _sequences.getUserFunctionByName)(props.functionName);

        if (_func) {
          // we need to check if there is a user function sequence
          var innerTasks = {};
          var componentTargetsStateMapping;

          if (events && events.length > 0) {
            events.forEach(function (innerEvent) {
              if (innerEvent && innerEvent.targets) {
                // select only user function targets
                var userFunctionTargets = innerEvent.targets.filter(function (innerEventTarget) {
                  return innerEventTarget.type === _constants.USER_FUNCTION_TYPE;
                });

                if (userFunctionTargets && userFunctionTargets.length > 0) {
                  innerTasks[innerEvent.name] = innerTasks[innerEvent.name] || [];
                  innerTasks[innerEvent.name] = [].concat(_toConsumableArray(innerTasks[innerEvent.name]), _toConsumableArray(createTasks(userFunctionTargets)));
                } // now find all connected components and get the instance key of the first one


                var componentTargets = innerEvent.targets.filter(function (innerEventTarget) {
                  return innerEventTarget.type === _constants.COMPONENT_TYPE;
                });

                if (componentTargets.length > 0) {
                  if (innerEvent.targets.length > 0) {
                    // we can pass only the first connected target state.
                    // It means that the connection should be only one
                    var innerEventTarget = innerEvent.targets[0];

                    if (innerEventTarget && innerEventTarget.props) {
                      var _innerEventTarget$pro = innerEventTarget.props,
                          componentName = _innerEventTarget$pro.componentName,
                          componentInstance = _innerEventTarget$pro.componentInstance;
                      componentTargetsStateMapping = componentTargetsStateMapping || {};
                      componentTargetsStateMapping[innerEvent.name] = "".concat(componentName, "_").concat(componentInstance);
                    }
                  }
                }
              }
            });
          } // create dispatchFunction in order to reuse its instance in the action function body


          var dispatchFunction = function dispatchFunction(functionKey, functionName, dispatchPayloads, dispatch, getState, helpers) {
            executeUserFunctionDispatch(events, innerTasks, functionKey, functionName, dispatchPayloads, dispatch, getState, helpers);
          }; // this function is used to pass the error object caught by the exception caching
          // the function is called with null error object before each user function invocation
          // this will let user to do not worry about the clearing of the error object


          var caughtExceptionFunction = function caughtExceptionFunction(functionKey, functionName, error, dispatch, getState, helpers) {
            executeUserFunctionDispatch(events, innerTasks, functionKey, functionName, _defineProperty({}, _constants.DISPATCH_ERROR_TYPE, error), dispatch, getState, helpers);

            if (error) {
              console.error("[Framework] In \"".concat(props.functionName, "\" function ").concat(error.message, "."));
            }
          }; // push function reference for user function dispatch


          tasks.push({
            functionKey: props.functionKey,
            funcName: props.functionName,
            func: function func() {
              var args = arguments;
              return function (dispatch, getState, helpers) {
                // before execute the function we have to transform the input argument if needed
                try {
                  var firstArgument = args[0]; // execute user function with passed in args

                  if (window.__webcodeskIsListeningToFramework && window.__sendFrameworkMessage) {
                    sendDebugMessage({
                      key: props.functionKey,
                      eventType: constants.DEBUG_MSG_FUNCTION_CALL_EVENT,
                      inputData: (0, _cloneDeep.default)(firstArgument),
                      functionName: props.functionName,
                      timestamp: Date.now()
                    });
                  }

                  var stateByDispatch;

                  if (componentTargetsStateMapping) {
                    stateByDispatch = {};
                    var currentGlobalState = getState();
                    (0, _forOwn.default)(componentTargetsStateMapping, function (value, key) {
                      if (value && value.length > 0) {
                        stateByDispatch[key] = currentGlobalState[value];
                      }
                    });
                  }

                  var userFunctionInstance = _func.apply(null, [firstArgument, {
                    stateByDispatch: stateByDispatch,
                    history: helpers.history
                  }]);

                  try {
                    // dispatch caughtException as null to the assigned targets
                    caughtExceptionFunction(props.functionKey, props.functionName, null, dispatch, getState, helpers); // now execute dispatching of the events objects to the targets

                    var userFunctionResult = userFunctionInstance(function (dispatchPayloads) {
                      // user function is invoked now
                      dispatchFunction(props.functionKey, props.functionName, dispatchPayloads, dispatch, getState, helpers);
                    }); // here user returns a Promise and there may be the error

                    if (userFunctionResult && userFunctionResult.then) {
                      userFunctionResult.catch(function (error) {
                        caughtExceptionFunction(props.functionKey, props.functionName, error, dispatch, getState, helpers);
                      });
                    }
                  } catch (error) {
                    caughtExceptionFunction(props.functionKey, props.functionName, error, dispatch, getState, helpers);
                  }
                } catch (e) {
                  console.error("[Framework] ".concat(e.message));
                }
              };
            }
          });
        } else {
          console.error("[Framework] Missing function: ".concat(props.functionName));
        }
      } else if (type === _constants.COMPONENT_TYPE && props) {
        tasks.push({
          func: function func() {
            var args = arguments; // arguments here is the passed parameters from the component event handler
            // we have to take only the first argument as a payload

            var payload = null;

            if (args.length > 0) {
              payload = args[0];
            }

            return function (dispatch, getState, helpers) {
              dispatchToComponent(props, payload, dispatch, helpers);
            };
          }
        });
      }
    });
  }

  return tasks;
}

function createActions(eventHandlers) {
  var actions = {};

  if (eventHandlers && eventHandlers.length > 0) {
    eventHandlers.forEach(function (eventHandler) {
      var name = eventHandler.name,
          targets = eventHandler.targets;
      var tasks = createTasks(targets);

      actions[name] = function () {
        var args = arguments;
        return function (dispatch, getState, helpers) {
          if (tasks.length > 0) {
            tasks.forEach(function (task) {
              if (task.func) {
                task.func.apply(null, args)(dispatch, getState, helpers);
              }
            });
          }
        };
      };
    });
  }

  return actions;
}

var actionsCache = new Map();
var transformFunctionsCache = new Map();

function clearActionsCache() {
  actionsCache.clear();
  transformFunctionsCache.clear();
}

function _default(eventHandlersKey, eventHandlers) {
  var actions = actionsCache.get(eventHandlersKey);

  if (!actions) {
    actions = createActions(eventHandlers);
    actionsCache.set(eventHandlersKey, actions);
  }

  return actions;
}

;