"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createActionSequences = createActionSequences;
exports.getUserFunctionByName = getUserFunctionByName;

var _uniqueId = _interopRequireDefault(require("lodash/uniqueId"));

var _forOwn = _interopRequireDefault(require("lodash/forOwn"));

var _get = _interopRequireDefault(require("lodash/get"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

var _isObject = _interopRequireDefault(require("lodash/isObject"));

var _isEmpty = _interopRequireDefault(require("lodash/isEmpty"));

var _unionWith = _interopRequireDefault(require("lodash/unionWith"));

var _isEqual = _interopRequireDefault(require("lodash/isEqual"));

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var userFunctions = {};

function getTargetsFromEvents(events, targetsMap) {
  if (events && events.length > 0) {
    events.forEach(function (event) {
      var targets = event.targets;

      if (targets && targets.length > 0) {
        targets.forEach(function (target) {
          var type = target.type,
              props = target.props,
              events = target.events;

          if (type === _constants.COMPONENT_TYPE && props) {
            var componentName = props.componentName,
                componentInstance = props.componentInstance;
            targetsMap["".concat(componentName, "_").concat(componentInstance)] = true;
          }

          getTargetsFromEvents(events, targetsMap);
        });
      }
    });
  }
}

function deriveTargets(actionSequences) {
  var targets = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  (0, _forOwn.default)(actionSequences, function (value, prop) {
    getTargetsFromEvents(value.events, targets);
  });
  return targets;
}

function getEventSequence(event) {
  var eventSequence = {};
  var name = event.name,
      targets = event.targets;

  if (targets && targets.length > 0) {
    eventSequence.name = name;
    eventSequence.targets = [];
    targets.forEach(function (target) {
      var type = target.type,
          props = target.props,
          events = target.events;

      if (props && !(0, _isEmpty.default)(props)) {
        var newTarget = {
          type: type
        };

        if (type === _constants.USER_FUNCTION_TYPE) {
          newTarget.props = _objectSpread({}, props, {
            functionKey: (0, _uniqueId.default)('seqNode')
          });
        } else if (type === _constants.COMPONENT_TYPE) {
          newTarget.props = _objectSpread({}, props, {
            componentKey: (0, _uniqueId.default)('seqNode')
          });
        }

        if (events && events.length > 0 && type === _constants.USER_FUNCTION_TYPE) {
          var newTargetEvents = [];
          events.forEach(function (targetEvent) {
            newTargetEvents.push(getEventSequence(targetEvent));
          });
          newTarget.events = newTargetEvents;
        }

        eventSequence.targets.push(newTarget);
      }
    });
  }

  return eventSequence;
}

function getTargetComparatorObjectForEvents(events) {
  var result = {};

  if (events && events.length > 0) {
    var sourceTargetEventTargets;

    for (var sei = 0; sei < events.length; sei++) {
      sourceTargetEventTargets = events[sei].targets;

      if (sourceTargetEventTargets && sourceTargetEventTargets.length > 0) {
        result[events[sei].name] = {};

        for (var seti = 0; seti < sourceTargetEventTargets.length; seti++) {
          var _sourceTargetEventTar = sourceTargetEventTargets[seti],
              type = _sourceTargetEventTar.type,
              props = _sourceTargetEventTar.props;

          if (type === _constants.COMPONENT_TYPE && props) {
            var componentName = props.componentName,
                componentInstance = props.componentInstance;
            result[events[sei].name]["".concat(componentName, "_").concat(componentInstance)] = true;
          }
        }
      }
    }
  }

  return result;
}

function eventTargetComparator(destTarget, sourceTarget) {
  var sourceType = sourceTarget.type,
      sourceProps = sourceTarget.props,
      sourceEvents = sourceTarget.events;
  var destType = destTarget.type,
      destProps = destTarget.props,
      destEvents = destTarget.events;
  var result = false;

  if (sourceType === destType) {
    if (sourceProps && destProps) {
      if (sourceProps.functionName && destProps.functionName) {
        var sourceTargetComparatorObject = getTargetComparatorObjectForEvents(sourceEvents);
        var destTargetComparatorObject = getTargetComparatorObjectForEvents(destEvents);
        result = sourceProps.functionName === destProps.functionName && (0, _isEqual.default)(destTargetComparatorObject, sourceTargetComparatorObject);
      } else if (sourceProps.componentName && destProps.componentName) {
        result = sourceProps.componentName === destProps.componentName && sourceProps.componentInstance === destProps.componentInstance;
      }
    }
  }

  return result;
}

function targetEventComparator(destEvent, sourceEvent) {
  return destEvent === sourceEvent || destEvent.name === sourceEvent.name;
}

function mergeTargetEvents(destEvents, sourceEvents) {
  var resultEvents = (0, _unionWith.default)(destEvents, sourceEvents, targetEventComparator);

  if (sourceEvents && sourceEvents.length > 0) {
    var foundDestEvent;
    sourceEvents.forEach(function (sourceEvent) {
      foundDestEvent = resultEvents.find(function (i) {
        return targetEventComparator(i, sourceEvent);
      });

      if (foundDestEvent) {
        foundDestEvent.targets = foundDestEvent.targets || [];
        sourceEvent.targets = sourceEvent.targets || [];
        foundDestEvent.targets = mergeEventTargets(foundDestEvent.targets, sourceEvent.targets);
      } else {
        resultEvents.push(sourceEvent);
      }
    });
  }

  return resultEvents;
}

function mergeEventTargets(destTargets, sourceTargets) {
  var resultTargets = (0, _unionWith.default)(destTargets, sourceTargets, eventTargetComparator);

  if (sourceTargets && sourceTargets.length > 0) {
    var foundDestTarget;
    sourceTargets.forEach(function (sourceTarget) {
      foundDestTarget = resultTargets.find(function (i) {
        return eventTargetComparator(i, sourceTarget);
      });

      if (foundDestTarget) {
        foundDestTarget.events = foundDestTarget.events || [];
        sourceTarget.events = sourceTarget.events || [];
        foundDestTarget.events = mergeTargetEvents(foundDestTarget.events, sourceTarget.events);
      } else {
        resultTargets.push(sourceTarget);
      }
    });
  }

  return resultTargets;
}

function getActionSequences(handlers) {
  var actionSequences = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (handlers && handlers.length > 0) {
    handlers.forEach(function (handler) {
      var type = handler.type,
          props = handler.props,
          events = handler.events;

      if (props && !props.isDisabled && events && events.length > 0) {
        events.forEach(function (event) {
          var key;
          var handlerObject;

          if (event && event.name && event.targets && event.targets.length > 0) {
            if (type === _constants.COMPONENT_TYPE) {
              key = "".concat(props.componentName, "_").concat(props.componentInstance);
              handlerObject = actionSequences[key] || _objectSpread({}, props, {
                componentKey: (0, _uniqueId.default)('seqNode'),
                events: []
              });
              var eventSequence = getEventSequence(event); // find the same event handler name for the container

              var existingHandlerEventIndex = handlerObject.events.findIndex(function (evn) {
                return evn.name === eventSequence.name;
              });

              if (existingHandlerEventIndex >= 0) {
                // here we should merge targets of the same container events handler
                var existingHandlerEvent = handlerObject.events[existingHandlerEventIndex];

                if (existingHandlerEvent) {
                  handlerObject.events[existingHandlerEventIndex].targets = mergeEventTargets(existingHandlerEvent.targets, eventSequence.targets);
                }
              } else {
                eventSequence.targets = mergeEventTargets(eventSequence.targets, eventSequence.targets);
                handlerObject.events.push(eventSequence);
              }

              actionSequences[key] = handlerObject;
            }

            actionSequences = getActionSequences(event.targets, actionSequences);
          }
        });
      }
    });
  }

  return actionSequences;
}

function createActionSequencesRecursively(handlers) {
  var actionSequences = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  (0, _forOwn.default)(handlers, function (value) {
    if ((0, _isArray.default)(value)) {
      // only arrays should be exported as flow sequences otherwise the object is assumed as a nested sequences
      actionSequences = getActionSequences(value, actionSequences);
    } else if ((0, _isObject.default)(value)) {
      // if the handlers is object - it means we have a nested handlers description
      actionSequences = createActionSequencesRecursively(value, actionSequences);
    }
  });
  return actionSequences;
}

function createActionSequences(handlers, functions) {
  userFunctions = _objectSpread({}, functions);
  var actionSequences = createActionSequencesRecursively(handlers);
  var targets = deriveTargets(actionSequences);
  return {
    actionSequences: actionSequences,
    targets: targets
  };
}

function getUserFunctionByName(functionName) {
  return (0, _get.default)(userFunctions, functionName);
}