"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createInitialState = createInitialState;

var _forOwn = _interopRequireDefault(require("lodash/forOwn"));

var _isArray = _interopRequireDefault(require("lodash/isArray"));

var _isObject = _interopRequireDefault(require("lodash/isObject"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createArrayState(arrayModel, initialState) {
  var result = undefined;

  if (arrayModel.length > 0) {
    var descriptionItem;
    var resultItem;

    for (var i = 0; i < arrayModel.length; i++) {
      descriptionItem = arrayModel[i];

      if (descriptionItem) {
        if ((0, _isArray.default)(descriptionItem)) {
          resultItem = createArrayState(descriptionItem, initialState);

          if (resultItem) {
            result = result || [];
            result.push(resultItem);
          }
        } else if ((0, _isObject.default)(descriptionItem)) {
          if (descriptionItem.type && descriptionItem.instance) {
            createComponentState(descriptionItem, initialState);
          } else {
            resultItem = createShapeState(descriptionItem, initialState);

            if (resultItem) {
              result = result || [];
              result.push(resultItem);
            }
          }
        } else {
          result = result || [];
          result.push(descriptionItem);
        }
      } else {
        result = result || [];
        result.push(descriptionItem);
      }
    }
  }

  return result;
}

function createShapeState(shapeModel, initialState) {
  var result;
  var resultItem;
  (0, _forOwn.default)(shapeModel, function (value, prop) {
    if (value) {
      if ((0, _isArray.default)(value)) {
        resultItem = createArrayState(value, initialState);

        if (resultItem) {
          result = result || {};
          result[prop] = resultItem;
        }
      } else if ((0, _isObject.default)(value)) {
        if (value.type && value.instance) {
          createComponentState(value, initialState);
        } else {
          resultItem = createShapeState(value, initialState);

          if (resultItem) {
            result = result || {};
            result[prop] = resultItem;
          }
        }
      } else {
        result = result || {};
        result[prop] = value;
      }
    } else {
      result = result || {};
      result[prop] = value;
    }
  });
  return result;
}

function createComponentState(componentModel, initialState) {
  if (componentModel) {
    var type = componentModel.type,
        instance = componentModel.instance,
        props = componentModel.props;

    if (props) {
      var key = "".concat(type, "_").concat(instance);
      initialState[key] = createShapeState(props, initialState);
    }
  }
}

function createInitialState(pages) {
  var initialState = {};

  if (pages) {
    (0, _forOwn.default)(pages, function (value) {
      createComponentState(value, initialState);
    });
  }

  return initialState;
}